import contextlib
import os
import subprocess
import sys
import tempfile


SCRIPT_PRE = r"""\documentclass[varwidth]{standalone}
\usepackage{graphicx}
\usepackage{url}
\begin{document}
"""
SCRIPT_POST = r"""\end{document}
"""


LATEXFILENAME = "master"


@contextlib.contextmanager
def tmpcd(dirname):
    savedir = os.getcwd()
    os.chdir(dirname)
    try:
        yield
    finally:
        os.chdir(savedir)


def main():
    graphicfilenames = sys.argv[1:]

    with tempfile.TemporaryDirectory() as tmpdirname:
        scriptfilename = os.path.join(tmpdirname, LATEXFILENAME + ".tex")
        mergedfilename = os.path.join(tmpdirname, LATEXFILENAME + ".pdf")

        with open(scriptfilename, "w") as scriptfile:
            scriptfile.write(SCRIPT_PRE)
            print(r"\graphicspath{{"+os.getcwd()+r"}}", file=scriptfile)
            for n, filename in enumerate(graphicfilenames):
                print(r"\path{"+filename+r"}", file=scriptfile)
                print(r"\linebreak", file=scriptfile)
                print(r"\includegraphics{"+filename+r"}", file=scriptfile)
                print(r"\linebreak", file=scriptfile)
            scriptfile.write(SCRIPT_POST)

        with tmpcd(tmpdirname):
            process = ["pdflatex", LATEXFILENAME]
            subprocess.call(process, stdout=sys.stderr)

        with open(mergedfilename, "br") as mergedfilename:
            sys.stdout.buffer.write(mergedfilename.read())

    return 0


if __name__ == "__main__":
    sys.exit(main())
