/* This program is part of a correctly-rounded, division-free l2
   vector norm with avoidance of spurious underflow and overflow.

   Author: Nicolas Louvet,

           Université Claude Bernard Lyon 1

   implementing the algorithm described in the paper

   Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
   Laurence Rideau. Accurate calculation of Euclidean Norms using
   Double-word arithmetic. 2021

   This program is

   Copyright (C) 2021 Université Claude Bernard Lyon 1

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.

*/

#include <gmp.h>
#include <mpfr.h>
#include "commons.h"

double netlibNorm(const double * x, unsigned int n);

double naiveNorm(const double * x, unsigned int n);

double vecNaiveNorm(const double * x, unsigned int n);

double faithfulNorm(const double * x, unsigned int n);

double faithfulNormNoOverflowFma(const double * x, unsigned int n);

double dummyFunc(const double * x, unsigned int);

double accurateNorm(const double *x, unsigned int n);
