/* This program is part of a faithfully-rounded, division-free l2
   vector norm with avoidance of spurious underflow and overflow.

   Author: Christoph Lauter,

           Université Pierre et Marie Curie Paris 6, UPMC, LIP6,
	   PEQUAN team.

   implementing the algorithm described in the paper

   Graillat, Lauter, Tang, Yamanaka and Oishi: Efficient calculations
   of faithfully rounded l2-norms of n-vectors.

   This program is

   Copyright (C) 2014 Université Pierre et Marie Curie Paris 6, UPMC,
   LIP6, PEQUAN team.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>. 

*/
/* Modified by: Nicolas Louvet,

           Université Claude Bernard Lyon 1

   implementing the algorithm described in the paper

   Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
   Laurence Rideau. Accurate calculation of Euclidean Norms using
   Double-word arithmetic. 2021

*/

#ifndef RANDOMVECTORS_H
#define RANDOMVECTORS_H

#include <gmp.h>
#include <mpfr.h>
#include "commons.h"

/* Some values for testing */
#define EXPO_MIN_ONE_RANGE     (-5)
#define EXPO_MAX_ONE_RANGE      (5)
/* #define EXPO_MAX_SMALL_RANGE (-400) */
#define EXPO_MAX_SMALL_RANGE (-512)

#define EXPO_MIN_MED -484
#define EXPO_MAX_MED +485

/* Some symbolic constants */
#define FULL_EXPONENT_RANGE        0
#define NO_OVERFLOW                1
#define AROUND_ONE                 2
#define REALLY_SMALL               3
#define NORM_GRADUALLY_UNDERFLOWS  4
#define HALF_ULP_CASES             5
#define MONOTONE_INCREASING        6
#define MONOTONE_DECREASING        7
#define NETLIB_SPURIOUS_UNDERFLOW  8

#define NRM_INP  10
#define SUB_SQR  11
#define SUB_INP  12
#define MED_INP  13
#define SPUR_OVF 14
#define INF_RES  15

void getNameOfTestCase(char name[25], int testCase);
void makeRandomVector(double *x, int n, int testCase);
int isFaithfulNorm(double y, CONST double * RESTRICT x, int n, int displayErrors, int *underflowCases, int *overflowCases, mpfr_t eps, mpfr_t ulpErr);


#endif /* RANDOMVECTORS_H */
