/* This program is part of a correctly-rounded, division-free l2
   vector norm with avoidance of spurious underflow and overflow.

   Author: Nicolas Louvet,

           Université Claude Bernard Lyon 1

   implementing the algorithm described in the paper

   Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
   Laurence Rideau. Accurate calculation of Euclidean Norms using
   Double-word arithmetic. 2021

   This program is

   Copyright (C) 2021 Université Claude Bernard Lyon 1

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.

*/

#if defined(__RDTSC__)

#pragma message("rdtsc")

#define READ_TIME_COUNTER(time)                        \
  __asm__ __volatile__(                                \
          "xorl %%eax,%%eax\n\t"                       \
          "cpuid\n\t"                                  \
          "rdtsc\n\t"                                  \
          "movl %%eax,(%0)\n\t"                        \
          "movl %%edx,4(%0)\n\t"                       \
          "xorl %%eax,%%eax\n\t"                       \
          "cpuid\n\t"                                  \
          : /* nothing */                              \
          : "S"((time))                                \
          : "eax", "ebx", "ecx", "edx", "memory")


#elif defined(__CLOCK_GETTIME__)

#pragma message("clock_gettime")
#include <time.h>

#define READ_TIME_COUNTER(time) do {                   \
  struct timespec t;                                   \
  clock_gettime(CLOCK_MONOTONIC, &t);                  \
  *time = (((uint64_t) t.tv_sec) *                     \
	   ((uint64_t) 1000000000)) +                        \
           ((uint64_t) t.tv_nsec);                     \
} while ( 0 );


#else

#pragma message("gettimeofday")
#define __GETTIMEOFDAY__
#include <time.h>

#define READ_TIME_COUNTER(time) do {                   \
  struct timeval t;                                    \
  gettimeofday(&t,NULL);                               \
  *time = (((uint64_t) t.tv_sec) *                     \
	   ((uint64_t) 1000000)) +                           \
           ((uint64_t) t.tv_usec);                     \
} while ( 0 );

#endif
