/* This program is part of a correctly-rounded, division-free l2
   vector norm with avoidance of spurious underflow and overflow.

   Author: Nicolas Louvet,

           Université Claude Bernard Lyon 1

   implementing the algorithm described in the paper

   Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
   Laurence Rideau. Accurate calculation of Euclidean Norms using
   Double-word arithmetic. 2021

   This program is

   Copyright (C) 2021 Université Claude Bernard Lyon 1

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.

*/

#include <math.h>
#include "vec.h"

/* basic vectorized version of the naive 2-norm algorithm */
double vecNaiveNorm(double *x, unsigned int n) {
    vec_t s_vec, x_vec;
    int i;

    s_vec = vec_set1(0.0);
    for(i = 0; i < n-vec_len; i+= vec_len) {
        x_vec = vec_load(x+i);
        s_vec = vec_fma(x_vec, x_vec, s_vec);
    }

    double s;
    for(s = 0.0; i < n; i++) s = fma(x[i], x[i], s);

    double VEC_ALGN s_tab[vec_len];
    vec_store_algn(s_tab, s_vec);

    for(i = 0; i < vec_len; i++) s += s_tab[i];

    return sqrt(s);
}
