How to use this code
====================

The directory EuNm-Julia is concieved a a Julia environment. To start using the
code, you should set this directory as your Julia project either by setting
the environment variable `JULIA_PROJECT`, or by using the `--project` option to
the `julia` command-line program. From this point, you must instantiate the
environment:

    $ julia --project
                   _
       _       _ _(_)_     |  Documentation: https://docs.julialang.org
      (_)     | (_) (_)    |
       _ _   _| |_  __ _   |  Type "?" for help, "]?" for Pkg help.
      | | | | | | |/ _` |  |
      | | |_| | | | (_| |  |  Version 1.5.3
     _/ |\__'_|_|_|\__'_|  |  Debian ⛬  julia/1.5.3+dfsg-3
    |__/                   |
    
    (EuNms) pkg> instantiate

(Remember to hit "]" in the Julia REPL to enter the package REPL.) Here is what
you can do:

* perform automatic tests to ensure your copy works as expected:
  `julia --project test/runtests.jl`. There is also *manual* check scripts,
  they start with `check_`.

* perform accuracy test by running scripts in the `accuracy` directory.

* perform benchmarks by running scripts in the `bench` directory.

* use the code interactively with `using EuNms` from within the Julia REPL.


How this code is organized
==========================

First, some tools:

* `dw.jl` to use double-word numbers more easily.
* `hyprand.jl`: an alternative random number generator.

Other tools are defined in `tools.jl`.

Algorithms of interest are in `code.jl`. They are organized as follows:

**DWAdd**
: Scalar addition algorithms of double-word numbers
:
: There is two algorithms: DWPlusFP, SloppyDWPlusDW.

**SQRTDW**
: Square root of a double-word number (SQRTDWtoFP, SQRTDWtoDW).

**DWSum**
: Vector addition algorithms of double-word numbers.
:
: They are based on DWAdd algorithms. So they are two algorithms labeled
: ``:plusfp`` and ``:sloppy``.

**SqSum**
: Vector addition algorithms of squared floating-point numbers.
:
: There are three algorithms: _nooverflow_ does not handle spurious overflows,
: whereas _masks_ and _onepass_ does; _masks_ splits the input into three
: categories, TINY, MED, and BIG, and handle them separately, and _onepass_
: does the same thing with only one pass through the vector.
:
: These algorithms are labeled ``:noovf``, ``:masks``, and ``:onep1``.

**EuNm**
: Euclidean norm of a vector of floating-point numbers. Combine the above
: algorithmes.

I hope I have made the code sufficiently self-explanatory.


Tricks
======

To print all Float64 in the %a format:

    Base.show(io::IO, x::Float64) = @printf io "%a" x


Conventions
===========

A comment starts with `## ` while commented code starts with `#`.
