## Author: Joris Picot,
##         École Normale Supérieure de Lyon, LIP, AriC Team.
##
## This program is part of an implementation of the algorithm described in the
## paper:
##
##     Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
##     Laurence Rideau. Accurate calculation of Euclidean Norms using
##     Double-word arithmetic. 2021
##
## Copyright (C) 2021  École Normale Supérieure de Lyon
##
## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.
##
## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
## USA

## Tools related to Graillat's faithfulnorm code.


const EXPO_MIN = -1074
const EXPO_MIN_ONE_RANGE = -5
const EXPO_MAX_ONE_RANGE =  5


randomDoubleMOneOne(rng::AbstractRNG, S::Type, ebnds) = copysign(hyprand(rng, S, ebnds), rand([-1,1]))
randomDoubleMOneOne(S::Type, ebnds) = randomDoubleMOneOne(Random.GLOBAL_RNG, S, ebnds)


rand_half_ulp_cases(rng::AbstractRNG, S::Type, size) = begin
    (β, _, _) = getFPparam(S)

    exactSquare = β^-33 + β^-53 - β^-12 * β^-53  ## 1.16415432822132222656641431512980489060282707214355e-10
    perturb = β^0/2 * β^-33 * β^-53 * β^10       ## 6.617444900424221398971269536559702828526496887207e-24
    [[one(S)]; [exactSquare + perturb * randomDoubleMOneOne(S, (EXPO_MIN, 0)) for _ in 1:size]]
end
rand_half_ulp_cases(S::Type, size) = rand_half_ulp_cases(Random.GLOBAL_RNG, S, size)
