## Author: Joris Picot,
##         École Normale Supérieure de Lyon, LIP, AriC Team.
##
## This program is part of an implementation of the algorithm described in the
## paper:
##
##     Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
##     Laurence Rideau. Accurate calculation of Euclidean Norms using
##     Double-word arithmetic. 2021
##
## Copyright (C) 2021  École Normale Supérieure de Lyon
##
## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.
##
## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
## USA

module EuNms


const WITH_BFLT = true   ## Activate BFloat16
const WITH_QUAD = true   ## Activate Float128
const WITH_SIMD = true   ## Activate SIMD
const WITH_ARB = true    ## Activate Arblib


## Default scale values.
const _DEFAULT_SCALE = Dict("BFloat16"=>80,
                            "Float32"=>100,
                            "Float64"=>700,
                            "Float128"=>10000)


using Printf
WITH_BFLT && using BFloat16s
WITH_QUAD && using Quadmath
WITH_SIMD && using SIMD
WITH_ARB && using Arblib


## Tools.
include("tools.jl")
include("dw.jl")
include("hyprand.jl")

WITH_BFLT && include("bfloat16.jl")
WITH_QUAD && include("float128.jl")
WITH_ARB && include("arblib.jl")

## Actual code.
include("code.jl")
WITH_SIMD && include("code_simd.jl")

## Post-process script.
include("tables.jl")
export Tables


end
