## Author: Joris Picot,
##         École Normale Supérieure de Lyon, LIP, AriC Team.
##
## This program is part of an implementation of the algorithm described in the
## paper:
##
##     Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
##     Laurence Rideau. Accurate calculation of Euclidean Norms using
##     Double-word arithmetic. 2021
##
## Copyright (C) 2021  École Normale Supérieure de Lyon
##
## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.
##
## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
## USA

export arb_is_unique


## Add ways to convert an Arb to floats (already defined for Float64).
for fmt in [:Float32]
    eval(
         :(Base.$fmt(x::Union{Arb,Arf}) = convert($fmt, BigFloat(x)))
        )
end
Float128(x::Union{Arb,Arf}) = convert(Float128, BigFloat(x))


Arb(x::Float128) = Arb(BigFloat(x))
## Can do better.


"""
    arb_is_unique(F, x)

tells wether `x` correspond to a unique floating-point number of format `F`.
"""
function arb_is_unique(F::Type{<:AbstractFloat}, x::Arb)
    (l, u) = convert.(F, getinterval(x))
    return l == u
end


is_faithful(value, exact::Arb) = is_faithful(value, BigFloat(exact))
is_correct(value, exact::Arb) = is_correct(value, BigFloat(exact))
