## Author: Joris Picot,
##         École Normale Supérieure de Lyon, LIP, AriC Team.
##
## This program is part of an implementation of the algorithm described in the
## paper:
##
##     Vincent Lefèvre, Nicolas Louvet, Jean-Michel Muller, Joris Picot,
##     Laurence Rideau. Accurate calculation of Euclidean Norms using
##     Double-word arithmetic. 2021
##
## Copyright (C) 2021  École Normale Supérieure de Lyon
##
## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.
##
## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
## USA

import Base: significand_bits, exponent_bits, exponent_bias, exponent_max, exponent_raw_max,
             significand_mask, exponent_mask, exponent_one, exponent_mask, significand_mask

significand_bits(::Type{Float128}) = trailing_ones(significand_mask(Float128))
exponent_bits(::Type{Float128}) = sizeof(Float128)*8 - significand_bits(Float128) - 1
exponent_bias(::Type{Float128}) = Int128(exponent_one(Float128) >> significand_bits(Float128))
# maximum float exponent
exponent_max(::Type{Float128}) = Int128(exponent_mask(Float128) >> significand_bits(Float128)) - exponent_bias(Float128)
# maximum float exponent without bias
exponent_raw_max(::Type{Float128}) = Int128(exponent_mask(Float128) >> significand_bits(Float128))

